/**
 * Created by haojunhua on 2017/1/17.
 */

import device from './device';
import biz from './biz';
import extra from './extra';
import storage from './storage';
import { noop, watch, ua, h5ApiToRN } from './utils';

var arenaApi = {
    device,
    biz,
    extra,
    storage,
    api:noop
};

var msgPre = 'arena.';

function connectWebViewJsBridge(callback) {
    if (window.WebViewJavascriptBridge) {
        callback(WebViewJavascriptBridge)
    } else {
        document.addEventListener(
            'WebViewJavascriptBridgeReady'
            , function () {
                callback(window.WebViewJavascriptBridge);
            },
            false
        );
    }
}

// 第一连接时初始化bridage
var initJsBridge = function () {
    if (ua.ios) {
        //ios 无需注册bridge

        (function () {
            if (window.WebViewJavascriptBridge) {
                return
            }
            window.WebViewJavascriptBridge = function () {
                var callbacks = [], callbackID = 0, registerHandlers = [];
                document.addEventListener("PacificDidReceiveNativeCallback", function (e) {
                    if (e.detail) {
                        var detail = e.detail;
                        var id = isNaN(parseInt(detail.id)) ? -1 : parseInt(detail.id);
                        if (id != -1) {
                            callbacks[id] && callbacks[id](detail.parameters, detail.error);
                            // delete callbacks[id]  //FIXME: 此处有坑，导致回调只能执行一次，如果想要多次执行回调，把该行代码注释掉即可，这是一种暴力解决的方案
                            delete callbacks[id] //上面的坑，在调用处重复调用来解决了
                        }
                    }
                }, false);
                document.addEventListener("PacificDidReceiveNativeBroadcast", function (e) {
                    if (e.detail) {
                        var detail = e.detail;
                        var name = detail.name;
                        if (name !== undefined && registerHandlers[name]) {
                            var namedListeners = registerHandlers[name];
                            if (namedListeners instanceof Array) {
                                var parameters = detail.parameters;
                                namedListeners.forEach(function (handler) {
                                    handler(parameters)
                                })
                            }
                        }
                    }
                }, false);
                return {
                    "post": function (action, parameters, callback, print) {
                        var id = callbackID++;
                        callbacks[id] = callback;
                        if (window.webkit && window.webkit.messageHandlers && window.webkit.messageHandlers.pacific) {
                            window.webkit.messageHandlers.pacific.postMessage({
                                "action": action,
                                "parameters": parameters,
                                "callback": id,
                                "print": print || 0
                            })
                        }
                    }, "on": function (name, callback) {
                        var namedListeners = registerHandlers[name];
                        if (!namedListeners) {
                            registerHandlers[name] = namedListeners = []
                        }
                        namedListeners.push(callback);
                        return function () {
                            namedListeners[indexOf(namedListeners, callback)] = null
                        }
                    }, "off": function (name) {
                        delete registerHandlers[name]
                    }
                }
            }()
        }());

        window.WebViewJavascriptBridge.on('nativeCallH5', function (parameters) {
            arenaApi.handlerNativeMsgFunc && arenaApi.handlerNativeMsgFunc(parameters);
        });

    } else if (ua.android && ua.h5App) {
        connectWebViewJsBridge(function (bridge) {
            //在客户端调用了mWebView.send("xxxx")时，会进入此方法，init相当于注册的一个默认的处理机制
            bridge.init(function (message, responseCallback) {
                var data = {
                    'data': message
                };
                responseCallback(data);
            });

            bridge.registerHandler("nativeCallH5", function (data, responseCallback) {

                console.log('accept msg : ' + data);

                console.log('handlerNativeMsgFunc: ' + arenaApi.handlerNativeMsgFunc);

                arenaApi.handlerNativeMsgFunc && arenaApi.handlerNativeMsgFunc(JSON.parse(data));

                console.log('handlerNativeMsgFunc  end ');

                var responseData = "回调数据" + data;
                responseCallback && responseCallback(responseData);
            });
        })
    }
}

initJsBridge();//执行初始化

function traverse(object, path, func) {
    for (var key in object) {
        var currentPath = path === "" ? key : path + "." + key
        var value = object[key]
        if (value !== noop) {
            traverse(value, currentPath, func)
        } else {
            func.apply(object, [currentPath, key])
        }
    }
}

function generator(method, key) {
    console.log(key);
    console.log(msgPre + method);

    method = msgPre + method;

    //此处相当于生成了所有的方法，业务直接调用了方法
    this[key] = function (param) {

        console.log(',,,', param);

        //console.log('调用方法：', method, '传参：', param);
        var p = param || {};
        var successCallback = function (res) {
            console.log('默认成功回调', method, res);
        };
        var failCallback = function (err) {
            console.log('默认失败回调', method, err)
        };
        var cancelCallback = function () {
        };
        if (p.onSuccess) {
            successCallback = p.onSuccess;
            delete p.onSuccess;
        }
        if (p.onFail) {
            failCallback = p.onFail;
            delete p.onFail;
        }

        //统一回调处理
        var callback = function (response) {
            console.log('response:' + response);

            var result = function (result) {
                if (ua.android) {
                    try {
                        return JSON.parse(result)
                    } catch (err) {
                        return result
                    }
                }
                return result
            }(response);

            var status = result.status;
            console.log('result:' + result);

            var data = "";
            try {
                data = JSON.parse(result.data);
            } catch (e) {
                data = result.data;
            }

            if (status >= 0) {
                successCallback && successCallback.call(null, data);
            } else {
                failCallback && failCallback.call(null, data);
                if (arenaApi.errorHandler) {
                    arenaApi.errorHandler(data)
                }
            }
        };

        //如果是h5应用
        if (ua.h5App) {

            if (ua.ios) {
                window.WebViewJavascriptBridge.post('ArenaMsg', {
                    'method': method,
                    'params': p,
                }, function (results, error) {
                    console.log('ios callback results： ', results);
                    console.log('ios callback error：', error);
                    if (results) {
                        successCallback(results);
                    } else if (error) {
                        failCallback(error);
                    }
                });

            } else {
                if (!window.WebViewJavascriptBridge) {
                    return console.log('WebViewJavascriptBridge undefined')
                }
                window.WebViewJavascriptBridge.callHandler(method, p, callback);
            }
        } else {//如果是RN应用
            //暂时不做处理
        }
    }

}

traverse(arenaApi, "", generator);

arenaApi.error = function (errorHandler) {
    if (!errorHandler) return
    arenaApi.errorHandler = errorHandler
}

arenaApi.handlerNativeMsg = function (callback) {
    arenaApi.handlerNativeMsgFunc = callback;
}

export default arenaApi